@@ -4,24 +4,39 @@ readPicture <- function(rgmlFile) {
     funGetX = function(x) { as.numeric(xmlAttrs(x)["x"]) }
     funGetY = function(x) { as.numeric(xmlAttrs(x)["y"]) }
     funGetGC = function(x) { pars <- xmlApply(x, xmlAttrs) }
-    funGetPath = function(x) { xmlAttrs(x)["type"] }
+    funPathType = function(x) { xmlAttrs(x)["type"] }
 
     funPath = function(x) {
         switch(xmlName(x),
                path={ xval = unlist(xmlApply(x, funGetX))
                       yval = unlist(xmlApply(x, funGetY))
-                      
+
                       # get context
                       gc = funGetGC(xmlElementsByTagName(x, "context")[[1]])
 
                       # convert to a list of S4 objects
-                      switch(funGetPath(x),
+                      switch(funPathType(x),
                              stroke=new("PictureStroke", x=xval, y=yval,
                                lwd=as.numeric(gc$style["lwd"]),
                                rgb=rgb(as.numeric(gc$rgb["r"]),
                                  as.numeric(gc$rgb["g"]),
                                  as.numeric(gc$rgb["b"]))),
                              fill=new("PictureFill", x=xval, y=yval,
+                               lwd=as.numeric(gc$style["lwd"]),
+                               rgb=rgb(as.numeric(gc$rgb["r"]),
+                                 as.numeric(gc$rgb["g"]),
+                                 as.numeric(gc$rgb["b"]))),
+                             text=new("PictureText",
+                               string=xmlAttrs(x)["string"],
+                               x=as.numeric(xmlAttrs(x)["x"]),
+                               y=as.numeric(xmlAttrs(x)["y"]),
+                               w=as.numeric(xmlAttrs(x)["width"]),
+                               h=as.numeric(xmlAttrs(x)["height"]),
+                               lwd=as.numeric(gc$style["lwd"]),
+                               rgb=rgb(as.numeric(gc$rgb["r"]),
+                                 as.numeric(gc$rgb["g"]),
+                                 as.numeric(gc$rgb["b"]))),
+                             char=new("PictureChar", x=xval, y=yval,
                                lwd=as.numeric(gc$style["lwd"]),
                                rgb=rgb(as.numeric(gc$rgb["r"]),
                                  as.numeric(gc$rgb["g"]),
@@ -47,8 +62,18 @@ readPicture <- function(rgmlFile) {
 pathBounds <- function(paths) {
     pathXmin <- function(path) { min(path@x) }
     pathYmin <- function(path) { min(path@y) }
-    pathXmax <- function(path) { max(path@x) }
-    pathYmax <- function(path) { max(path@y) }
+    pathXmax <- function(path) {
+        if (is(path, "PictureText"))
+            path@x + path@w
+        else
+            max(path@x)
+    }
+    pathYmax <- function(path) {
+        if (is(path, "PictureText"))
+            path@y + path@h
+        else
+            max(path@y)
+    }
     list(xscale=c(min(sapply(paths, pathXmin)),
            max(sapply(paths, pathXmax))),
          yscale=c(min(sapply(paths, pathYmin)), 
